import codecs
import datetime
import re
import urllib2

from BeautifulSoup import BeautifulSoup

NOSCR = re.compile("<script .*?</script>", re.DOTALL)

def sanitize(text):
    """Remove scripts and do some transformation."""
    text = NOSCR.sub("", text)
    text = text.replace("&nbsp;", " ")
    text = text.replace('target="_blank"', "")
    return text


def parse_player_page(url, drop_date):
    """Parse player page and get the next losing score."""
    page = url.read()

    # sanitize and parse
    page = sanitize(page)
    with open("/tmp/lastone", "w") as fh:
        fh.write(page)
    soup = BeautifulSoup(page)

    todos_puntos = []
    for seccion in soup.findAll('h5', {'class': "profileSecondaryTitleAlt3"}):
        if seccion.string == u'Non-Countable Tournaments':
            # these points don't count
            continue

        tabla = seccion.next.next.next
        first_one = True
        for row in tabla.findAll('tr'):
            data = []
            for td in row.findAll('td'):
                c = td.contents
                data.append(c[0] if c else None)

            if first_one:
                first_one = False
                assert data[3] == u'Points'
                assert data[4] == u'Drop Date'
                continue

            puntos = int(data[3].replace(",", ""))
            if data[4] is not None:
                d, m, y = map(int, data[4].split("."))
                fecha = datetime.date(y, m, d)
            else:
                fecha = datetime.date(2999, 1, 1)
            todos_puntos.append((fecha, puntos))

    todos_puntos.sort()
    ranking_score = sum(x[1] for x in todos_puntos)
    prox_fecha, drop = todos_puntos[0]
    if prox_fecha != drop_date:
        drop = 0
    return ranking_score, drop


def parse_ranking_page(url, drop_date):
    """Parse the ranking page and get all info."""
    page = url.read()

    # sanitize and parse
    page = sanitize(page)
    soup = BeautifulSoup(page)

    full_table = soup.find('div', {'class': "bioTableWrap bioTableWrapAlt"})
    first_one = True
    for row in full_table.findAll('tr'):
        data = [x.contents[0] for x in row.findAll('td')]
        if first_one:
            first_one = False
            assert data[0] == u'Rank'
            assert data[1] == u'Name & Nationality'
            assert data[2] == u'Points'
            continue

        # rank and name are easy
        rank = int(data[0])
        name = data[1].string
        print "%d. %s" % (rank, name)

        # link is tough
        p = data[2]
        points = int(p.string.replace(",", ""))
        link = "http://www.atpworldtour.com" + p.attrs[0][1] + "&d=n"

#        print rank, repr(name), points, link

        u = urllib2.urlopen(link)
        (ranking_score, dropping) = parse_player_page(u, drop_date)
        if ranking_score != points:
            raise ValueError("no matchea points")
#        print "    drop", dropping
        yield rank, name, points, dropping


def main():
    # get next Monday (if not Monday today)
    day = datetime.date.today()
    while day.weekday() != 0:
        day += datetime.timedelta(days=1)
    print "Next Monday", day

    pags = [
        "http://www.atpworldtour.com/Rankings/Singles.aspx?r=1",
        "http://www.atpworldtour.com/Rankings/Singles.aspx?r=101",
    ]
    rank_shouldbe = 0
    all_data = []
    for pag in pags:
        u = urllib2.urlopen(pag)
        for user_data in parse_ranking_page(u, day):
            rank_shouldbe += 1
            rank, name, points, drop = user_data
            if rank_shouldbe != rank:
                raise ValueError("Ranking mezclado!")
            all_data.append((name, points, drop))

    # get maximums
    max_name = max(len(x[0]) for x in all_data)
    max_pnts = max(len(str(x[1])) for x in all_data)
    max_drop = max(len(str(x[2])) for x in all_data)

    # save to the file
    with codecs.open("players.txt", "w", "utf8") as fh:
        for name, points, drop in all_data:
            t = "%-*s : %*d : %*d" % (max_name, name, max_pnts, points,
                                     max_drop, drop)
            fh.write(t + "\n")

if __name__ == "__main__":
    main()
#    u = open("/tmp/Rafael-Nadal.aspx.html")
#    print parse_player_page(u, datetime.date.today())
